@@ -1,4 +1,4 @@
-# Copyrights 2014 by [Mark Overmeer].
+# Copyrights 2014-2015 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::WSDL11;
 use vars '$VERSION';
-$VERSION = '3.03';
+$VERSION = '3.04';
 
 use base 'XML::Compile::Cache';
 
@@ -138,6 +138,11 @@ sub addWSDL($%)
 {   my ($self, $data, %args) = @_;
     defined $data or return ();
 
+    if(ref $data eq 'ARRAY')
+    {   $self->addWSDL($_) for @$data;
+        return $self;
+    }
+
     my ($node, %details) = $self->dataToXML($data);
     defined $node or return $self;
 
@@ -166,7 +171,7 @@ sub addWSDL($%)
 
         if($which eq 'service')
         {   foreach my $port ( @{$def->{wsdl_port} || []} )
-            {   my $addr_label = first { /_address$/ } keys %$port
+            {   my $addr_label = first { /address$/ } keys %$port
                     or error __x"no address in port {port}"
                         , port => $port->{name};
                 my $first_addr = $port->{$addr_label};
@@ -174,7 +179,7 @@ sub addWSDL($%)
 
                 # Is XML::Compile::SOAP<version> loaded?
                 ref $first_addr eq 'HASH'
-                    or error __x"soap namespace {ns} not loaded"
+                    or error __x"wsdl namespace {ns} not loaded"
                        , ns => $first_addr->namespaceURI;
 
                 $index->{port}{pack_type $tns, $port->{name}} = $port;
@@ -464,8 +469,8 @@ sub operations(@)
     my @ops;
     my @services = $self->findDef('service');
     foreach my $service (@services)
-    {
-        next if $args{service} && $args{service} ne $service->{name};
+    {   my $sname = $service->{name};
+        next if $args{service} && $args{service} ne $sname;
 
         my @ports = @{$service->{wsdl_port} || []};
         foreach my $port (@ports)
@@ -484,22 +489,27 @@ sub operations(@)
 
             my %all_ops;
             foreach my $operation ( @{$binding->{wsdl_operation}||[]} )
-            {   my $name = $operation->{name};
-                if($all_ops{$name}++)
-                {   panic __x"operation {name} found again; pick service from {services}"
-                      , services => [map $_->{name}, @services], _join => ', '
+            {   my $opname = $operation->{name};
+
+                if(my $has = $all_ops{$opname})
+                {   error __x"operation {name} found again; choose service {has} or {also}"
+                      , name => $opname, has => $has->serviceName
+                      , also => $sname
                         if @services > 1 && !$args{service};
-                    panic __x"need one set of operations, pick port from {ports}"
+
+                    error __x"need one set of operations, pick port from {ports}"
                        , ports => [ map $_->{name}, @ports ], _join => ', ';
                 }
   
-                push @ops, $self->operation
-                  ( service   => $service->{name}
+                my $op = $all_ops{$opname} = $self->operation
+                  ( service   => $sname
                   , port      => $port->{name}
                   , binding   => $bindtype
-                  , operation => $name
+                  , operation => $opname
                   , portType  => $type
                   );
+
+                push @ops, $op;
             }
         }
     }
@@ -514,7 +524,7 @@ sub endPoint(@)
 
     my $port;
     my @ports     = @{$service->{wsdl_port} || []};
-    my @portnames = map {$_->{name}} @ports;
+    my @portnames = map $_->{name}, @ports;
     if(my $portname = delete $args{port})
     {   $port = first {$_->{name} eq $portname} @ports;
         error __x"cannot find port `{portname}', pick from {ports}"
